//Miniaturowy przypominacz AVT
//Micha Kurzela @ 2018

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#define CZAS_1 15 //definicja dugoci odmierzanych czasw [w minutach]
#define CZAS_2 30
#define CZAS_3 60
#define CZAS_4 120

//progi ADC do wykrycia zworek
#define PROG_4 700 //nominalnie 512
#define PROG_3 380 //nominalnie 327
#define PROG_2 250 //nominalnie 185
#define PROG_1 130 //nominalnie 93

#define LADDER_ON PORTB|=(1<<PORTB1)
#define LADDER_OFF PORTB&=~(1<<PORTB1)

#define BUZZER_ON PORTB|=(1<<PORTB3)
#define BUZZER_OFF PORTB&=~(1<<PORTB3)

#define LED_ON PORTB|=(1<<PORTB4)
#define LED_OFF PORTB&=~(1<<PORTB4)

#define LICZBA_BLYSKOW 4 //liczba byskw w jednym cyklu
#define PRZERWA_KROTKA 50 //dugo krtkiej przerwy w generowaniu byskw
#define PRZERWA_DLUGA 250 //dugo przerw midzy seriami byskw

volatile uint8_t sekundy = 0; //odliczone sekundy
volatile uint8_t minuty = 0; //odliczone minuty
volatile uint8_t czy_odlicza = 0; //0 - pozostaje upiony, 1 - odlicza, 2 - wszczyna alarm
volatile uint8_t czas_zadany; //zadany do odliczenia czas w minutach

//---------------------------------------------------------------   
// Przerwanie od przetwornika A/C                                    
//---------------------------------------------------------------
ISR(ADC_vect){
if((ADCL | (ADCH<<8)) <= PROG_1){ czas_zadany = CZAS_1; }
if((ADCL | (ADCH<<8)) > PROG_1 && (ADCL | (ADCH<<8)) <= PROG_2){ czas_zadany = CZAS_2; }
if((ADCL | (ADCH<<8)) > PROG_2 && (ADCL | (ADCH<<8)) <= PROG_3){ czas_zadany = CZAS_3; }
if((ADCL | (ADCH<<8)) > PROG_3 && (ADCL | (ADCH<<8)) <= PROG_4){ czas_zadany = CZAS_4; }
if((ADCL | (ADCH<<8)) > PROG_4){ czas_zadany = 0; czy_odlicza = 0;} //przejd w stan upienia, jeeli nie ma zaczonego DIP switcha
ADCSRA = 0; LADDER_OFF; //wycz ADC i drabink

if(czy_odlicza){ //jeeli odliczanie nie zostao zatrzymane
sekundy = 0; minuty = 0; //przygotowanie do odliczania
WDTCR = (1 << WDTIE) | (1 << WDP1) | (1 << WDP2); //uruchomienie przerwa od watchdoga + timeout 1s
}
LED_OFF; //wycz diody LED - uytkownik otrzyma mignicie

}

//---------------------------------------------------------------   
// Przerwanie od przycisku SW1                                     
//---------------------------------------------------------------
ISR(PCINT0_vect){
if(bit_is_clear(PINB,PINB0)){

if(czy_odlicza == 0){ //jeeli nie odlicza
LADDER_ON; //zaczenie drabinki rezystorowej
LED_ON; //zaczenie diod LED dla uzyskania mignicia potwierdzajcego start
czy_odlicza = 1; //trwa odliczanie
ADMUX = (1 << MUX0); //wejcie ADC1
ADCSRA = (1 << ADEN) | (1 << ADATE) | (1 << ADIE) | (1 << ADPS2); //uruchomienie ADC, wielokrotne powtarzanie, uruchomienie przerwania, preskaler przez 16
ADCSRA |= (1 << ADSC); //rozpoczcie konwersji
_delay_ms(200); //aby wyeliminowa drganie stykw
}

if(czy_odlicza == 2){ //jeeli zosta wszczty alarm po odliczeniu czasu
czy_odlicza = 0; //przejcie w stan upienia
WDTCR = 0; //watchdog jest ju niepotrzebny
LED_OFF; BUZZER_OFF; //wycz diody i buzzer
_delay_ms(1000); //aby wyeliminowa drganie stykw
}

}}

//---------------------------------------------------------------  
// Przerwanie od Watchdog                                      
//---------------------------------------------------------------
ISR(WDT_vect){ 

if(czy_odlicza == 1){
sekundy++;
LED_ON;
_delay_ms(2);
LED_OFF;
}
if(sekundy >= 60){minuty++; sekundy = 0; }
if(minuty >= czas_zadany){ czy_odlicza = 2;} //rozpocznij alarm

} 


//---------------------------------------------------------------
// GLWNA FUNKCJA PROGRAMU                                                   
//---------------------------------------------------------------

int main(void)
{

//konfiguracja wyprowadze 
DDRB = (1 << DDB4) | (1 << DDB3) | (1 << DDB1); //wyjcia LED, BUZZER i LADDER_ON
PORTB = (1 << PORTB0); //wejcie SWITCH w stanie wysokim


//uruchomienie przerwa od przycisku
GIMSK = (1 << PCIE); //aktywacja systemu przerwa PCINT
PCMSK = (1 << PCINT0); //przerwanie od pinu PINB0

sei(); //uruchomienie systemu przerwa


while(1){

	while(czy_odlicza == 2){ //w stanie alarmu 
	for(uint8_t i = 0 ; i < LICZBA_BLYSKOW ; i++){
	LED_ON; BUZZER_ON;
	_delay_ms(PRZERWA_KROTKA);
	LED_OFF; BUZZER_OFF;
	_delay_ms(PRZERWA_KROTKA);
	}
	_delay_ms(PRZERWA_DLUGA);
	}


set_sleep_mode(SLEEP_MODE_PWR_DOWN);
sleep_enable();
sleep_cpu(); 
}

sleep_disable();




}
